==> core-android-debug-1.19.16-sources.jar: commonMain/io/dyte/core/socket/socketservice/SocketService.kt <==
package io.dyte.core.socket.socketservice

import io.dyte.core.observability.DyteLogger
import io.dyte.sockrates.client.*
import io.dyte.sockrates.client.SockratesResult.Failure
import io.dyte.sockrates.client.SockratesResult.Success
import kotlin.coroutines.cancellation.CancellationException
import kotlin.random.Random
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import message.v1.SocketMessage

interface SocketService {
  suspend fun connect()

  suspend fun disconnect()

  suspend fun send(event: Int, payload: ByteArray?, messageId: String? = null)

  @Throws(ResponseTimeoutException::class, CancellationException::class)
  suspend fun requestResponse(
    event: Int,
    payload: ByteArray?,
    messageId: String? = null
  ): ByteArray?

  fun subscribe(event: Int, listener: SocketServiceEventListener)

  fun unsubscribe(event: Int, listener: SocketServiceEventListener)

  fun addConnectionStateListener(listener: SocketServiceConnectionStateListener)

  fun removeConnectionStateListener(listener: SocketServiceConnectionStateListener)

  fun clear()
}

interface SocketServiceEventListener {
  fun onEvent(event: Int, eventId: String?, payload: ByteArray?)
}

interface SocketServiceConnectionStateListener {
  fun onConnectionStateChanged(newState: SocketServiceConnectionState)
}

class ResponseTimeoutException(override val message: String) : RuntimeException(message)

internal class DefaultSocketService(
  baseUrl: String,
  private val peerId: String,
  roomName: String,
  authToken: String,
  useHive: Boolean,
  private val workContext: CoroutineDispatcher = Dispatchers.Default,
) : SocketService {
  private val url: String
  private val wsClient: Sockrates
  // private val mainScope = MainScope()

  private val activeEventListeners: MutableMap<Int, MutableSet<SocketServiceEventListener>> =
    HashMap()

  private val connectionStateListeners: MutableSet<SocketServiceConnectionStateListener> =
    LinkedHashSet()

  init {
    url = createSocketServiceUrl(baseUrl, peerId, roomName, authToken, useHive)
    wsClient = Sockrates(url, config = SockratesConfiguration(disconnectOnPingTimeout = false))
  }

  override suspend fun connect() {
    withContext(workContext) {
      val result =
        wsClient.connect(
          object : SockratesWSListener {
            override fun onConnectionStateChanged(
              client: Sockrates,
              newState: WebSocketConnectionState
            ) {
              super.onConnectionStateChanged(client, newState)
              try {
                val socketServiceConnectionState =
                  SocketServiceConnectionState.createFromSockratesConnectionState(newState)
                notifyConnectionStateListeners(socketServiceConnectionState)
              } catch (e: IllegalArgumentException) {
                // no-op
              }
            }

            override fun onMessage(client: Sockrates, message: SocketMessage) {
              super.onMessage(client, message)
              notifyEventSubscribers(message)
            }
          }
        )

      when (result) {
        is Failure -> {
          DyteLogger.error("SocketService::connect::failed ${result.reason}")
        }
        is Success -> {
          DyteLogger.info("SocketService::connect::success")
        }
      }
    }
  }

  override suspend fun disconnect() {
    withContext(workContext) { wsClient.disconnect() }
  }

  override fun clear() {
    activeEventListeners.clear()
    connectionStateListeners.clear()
    wsClient.clear()
  }

  override suspend fun send(event: Int, payload: ByteArray?, messageId: String?) {
    withContext(workContext) {
      val result =
        wsClient.send(
          event = event,
          messageId = messageId ?: generateMessageId(peerId),
          payload = payload
        )

      when (result) {
        is Failure -> {
          DyteLogger.error("SocketService::send event $event::failed ${result.reason}")
        }
        is Success -> {
          DyteLogger.info("SocketService::send event $event::success")
        }
      }
    }
  }

  override suspend fun requestResponse(
    event: Int,
    payload: ByteArray?,
    messageId: String?
  ): ByteArray? {
    return withContext(workContext) {
      val result =
        wsClient.requestResponse(
          event = event,
          messageId = messageId ?: generateMessageId(peerId),
          payload = payload
        )
      when (result) {
        is Success -> {
          DyteLogger.info("SocketService::requestResponse event $event::success")
          return@withContext result.value.payload?.toByteArray()
        }
        is Failure -> {
          val reason = result.reason
          DyteLogger.error("SocketService::requestResponse event $event::failure $reason")
          when (reason) {
            is RequestResponseFailureReason.ResponseTimeout -> {
              throw ResponseTimeoutException(
                "SocketService response timeout after ${reason.timeoutInMillis}ms"
              )
            }
            is RequestResponseFailureReason.SocketNotConnected -> {
              return@withContext null
            }
            is RequestResponseFailureReason.Other -> {
              return@withContext null
            }
          }
        }
      }
    }
  }

  override fun subscribe(event: Int, listener: SocketServiceEventListener) {
    val existingEventListeners = activeEventListeners[event]
    if (existingEventListeners == null) {
      val newEventListenersSet = LinkedHashSet<SocketServiceEventListener>()
      newEventListenersSet.add(listener)
      activeEventListeners[event] = newEventListenersSet
    } else {
      existingEventListeners.add(listener)
    }
  }

  override fun unsubscribe(event: Int, listener: SocketServiceEventListener) {
    val existingEventListeners = activeEventListeners[event]
    if (existingEventListeners != null) {
      existingEventListeners.remove(listener)
    }
  }

  override fun addConnectionStateListener(listener: SocketServiceConnectionStateListener) {
    connectionStateListeners.add(listener)
  }

  override fun removeConnectionStateListener(listener: SocketServiceConnectionStateListener) {
    connectionStateListeners.remove(listener)
  }

  private fun notifyEventSubscribers(socketMessage: SocketMessage) {
    if (activeEventListeners.isEmpty()) return

    val eventListeners = activeEventListeners[socketMessage.event]
    if (eventListeners != null && eventListeners.isNotEmpty()) {
      for (listener in eventListeners) {
        listener.onEvent(
          socketMessage.event,
          socketMessage.id,
          socketMessage.payload?.toByteArray()
        )
      }
    }
  }

  private fun notifyConnectionStateListeners(connectionState: SocketServiceConnectionState) {
    connectionStateListeners.forEach { it.onConnectionStateChanged(connectionState) }
  }

  companion object {
    private fun createSocketServiceUrl(
      baseUrl: String,
      peerId: String,
      roomName: String,
      authToken: String,
      useHive: Boolean,
    ): String {
      return "$baseUrl/ws?roomID=${roomName}&peerID=${peerId}&authToken=${authToken}&useMediaV2=${useHive}"
    }

    private fun generateMessageId(peerId: String): String {
      val randomSuffix = generateRandomString(5)
      return "${peerId}-${randomSuffix}"
    }

    private val characterSet =
      charArrayOf(
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9',
        'a',
        'b',
        'c',
        'd',
        'e',
        'f',
        'g',
        'h',
        'i',
        'j',
        'k',
        'l',
        'm',
        'n',
        'o',
        'p',
        'q',
        'r',
        's',
        't',
        'u',
        'v',
        'w',
        'x',
        'y',
        'z'
      )

    /*
    This is not finalised yet. Currently using the most straightforward method to
    generate a random 5 character alphanumeric string.
    Will need to come-up with a better efficient method or implement Base36 encoding in
    Kotlin.
     */
    private fun generateRandomString(idLength: Int = 5): String {
      val idBuilder: StringBuilder = StringBuilder()
      var randomIndex = 0
      for (i in 0 until idLength) {
        randomIndex = (Random.nextInt(until = characterSet.size))
        idBuilder.append(characterSet[randomIndex])
      }

      return idBuilder.toString()
    }
  }
}

==> core-android-debug-1.21.0-sources.jar: commonMain/io/dyte/core/socket/socketservice/SocketService.kt <==
package io.dyte.core.socket.socketservice

import io.dyte.core.observability.DyteLogger
import io.dyte.sockrates.client.*
import io.dyte.sockrates.client.SockratesResult.Failure
import io.dyte.sockrates.client.SockratesResult.Success
import kotlin.coroutines.cancellation.CancellationException
import kotlin.random.Random
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import message.v1.SocketMessage

interface SocketService {
  suspend fun connect()

  suspend fun disconnect()

  suspend fun send(event: Int, payload: ByteArray?, messageId: String? = null)

  @Throws(ResponseTimeoutException::class, CancellationException::class)
  suspend fun requestResponse(
    event: Int,
    payload: ByteArray?,
    messageId: String? = null
  ): ByteArray?

  fun subscribe(event: Int, listener: SocketServiceEventListener)

  fun unsubscribe(event: Int, listener: SocketServiceEventListener)

  fun addConnectionStateListener(listener: SocketServiceConnectionStateListener)

  fun removeConnectionStateListener(listener: SocketServiceConnectionStateListener)

  fun clear()

  /**
   * Generates a new socket-service URL with the provided peerId.
   *
   * **NOTE**: This is needed as a part of a workaround that we have done to make the socket-service
   * work properly after the reconnection.
   */
  fun refreshUrl(peerId: String)
}

interface SocketServiceEventListener {
  fun onEvent(event: Int, eventId: String?, payload: ByteArray?)
}

interface SocketServiceConnectionStateListener {
  fun onConnectionStateChanged(newState: SocketServiceConnectionState)
}

class ResponseTimeoutException(override val message: String) : RuntimeException(message)

internal class DefaultSocketService(
  private val baseUrl: String,
  private var peerId: String,
  private val roomName: String,
  private val authToken: String,
  private val useHive: Boolean,
  private val workContext: CoroutineDispatcher = Dispatchers.Default,
) : SocketService {
  private var url: String
  private var wsClient: Sockrates
  // private val mainScope = MainScope()

  private val activeEventListeners: MutableMap<Int, MutableSet<SocketServiceEventListener>> =
    HashMap()

  private val connectionStateListeners: MutableSet<SocketServiceConnectionStateListener> =
    LinkedHashSet()

  init {
    url = createSocketServiceUrl(baseUrl, peerId, roomName, authToken, useHive)
    wsClient =
      Sockrates(
        url,
        config = SockratesConfiguration(disconnectOnPingTimeout = false, autoReconnect = false)
      )
  }

  override suspend fun connect() {
    withContext(workContext) {
      val result =
        wsClient.connect(
          object : SockratesWSListener {
            override fun onConnectionStateChanged(
              client: Sockrates,
              newState: WebSocketConnectionState
            ) {
              super.onConnectionStateChanged(client, newState)
              try {
                val socketServiceConnectionState =
                  SocketServiceConnectionState.createFromSockratesConnectionState(newState)
                notifyConnectionStateListeners(socketServiceConnectionState)
              } catch (e: IllegalArgumentException) {
                // no-op
              }
            }

            override fun onMessage(client: Sockrates, message: SocketMessage) {
              super.onMessage(client, message)
              notifyEventSubscribers(message)
            }
          }
        )

      when (result) {
        is Failure -> {
          DyteLogger.error("SocketService::connect::failed ${result.reason}")
        }
        is Success -> {
          DyteLogger.info("SocketService::connect::success")
        }
      }
    }
  }

  override suspend fun disconnect() {
    withContext(workContext) { wsClient.disconnect() }
  }

  override fun clear() {
    activeEventListeners.clear()
    connectionStateListeners.clear()
    wsClient.clear()
  }

  override suspend fun send(event: Int, payload: ByteArray?, messageId: String?) {
    withContext(workContext) {
      val result =
        wsClient.send(
          event = event,
          messageId = messageId ?: generateMessageId(peerId),
          payload = payload
        )

      when (result) {
        is Failure -> {
          DyteLogger.error("SocketService::send event $event::failed ${result.reason}")
        }
        is Success -> {
          DyteLogger.info("SocketService::send event $event::success")
        }
      }
    }
  }

  override suspend fun requestResponse(
    event: Int,
    payload: ByteArray?,
    messageId: String?
  ): ByteArray? {
    return withContext(workContext) {
      val result =
        wsClient.requestResponse(
          event = event,
          messageId = messageId ?: generateMessageId(peerId),
          payload = payload
        )
      when (result) {
        is Success -> {
          DyteLogger.info("SocketService::requestResponse event $event::success")
          return@withContext result.value.payload?.toByteArray()
        }
        is Failure -> {
          val reason = result.reason
          DyteLogger.error("SocketService::requestResponse event $event::failure $reason")
          when (reason) {
            is RequestResponseFailureReason.ResponseTimeout -> {
              throw ResponseTimeoutException(
                "SocketService response timeout after ${reason.timeoutInMillis}ms"
              )
            }
            is RequestResponseFailureReason.SocketNotConnected -> {
              return@withContext null
            }
            is RequestResponseFailureReason.Other -> {
              return@withContext null
            }
          }
        }
      }
    }
  }

  override fun subscribe(event: Int, listener: SocketServiceEventListener) {
    val existingEventListeners = activeEventListeners[event]
    if (existingEventListeners == null) {
      val newEventListenersSet = LinkedHashSet<SocketServiceEventListener>()
      newEventListenersSet.add(listener)
      activeEventListeners[event] = newEventListenersSet
    } else {
      existingEventListeners.add(listener)
    }
  }

  override fun unsubscribe(event: Int, listener: SocketServiceEventListener) {
    val existingEventListeners = activeEventListeners[event]
    if (existingEventListeners != null) {
      existingEventListeners.remove(listener)
    }
  }

  override fun addConnectionStateListener(listener: SocketServiceConnectionStateListener) {
    connectionStateListeners.add(listener)
  }

  override fun removeConnectionStateListener(listener: SocketServiceConnectionStateListener) {
    connectionStateListeners.remove(listener)
  }

  override fun refreshUrl(peerId: String) {
    this.peerId = peerId
    url = createSocketServiceUrl(baseUrl, peerId, roomName, authToken, useHive)
    wsClient =
      Sockrates(
        url,
        config = SockratesConfiguration(disconnectOnPingTimeout = false, autoReconnect = false)
      )
  }

  private fun notifyEventSubscribers(socketMessage: SocketMessage) {
    if (activeEventListeners.isEmpty()) return

    val eventListeners = activeEventListeners[socketMessage.event]
    if (eventListeners != null && eventListeners.isNotEmpty()) {
      for (listener in eventListeners) {
        listener.onEvent(
          socketMessage.event,
          socketMessage.id,
          socketMessage.payload?.toByteArray()
        )
      }
    }
  }

  private fun notifyConnectionStateListeners(connectionState: SocketServiceConnectionState) {
    connectionStateListeners.forEach { it.onConnectionStateChanged(connectionState) }
  }

  companion object {
    private fun createSocketServiceUrl(
      baseUrl: String,
      peerId: String,
      roomName: String,
      authToken: String,
      useHive: Boolean,
    ): String {
      return "$baseUrl/ws?roomID=${roomName}&peerID=${peerId}&authToken=${authToken}&useMediaV2=${useHive}"
    }

    private fun generateMessageId(peerId: String): String {
      val randomSuffix = generateRandomString(5)
      return "${peerId}-${randomSuffix}"
    }

    private val characterSet =
      charArrayOf(
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9',
        'a',
        'b',
        'c',
        'd',
        'e',
        'f',
        'g',
        'h',
        'i',
        'j',
        'k',
        'l',
        'm',
        'n',
        'o',
        'p',
        'q',
        'r',
        's',
        't',
        'u',
        'v',
        'w',
        'x',
        'y',
        'z'
      )

    /*
    This is not finalised yet. Currently using the most straightforward method to
    generate a random 5 character alphanumeric string.
    Will need to come-up with a better efficient method or implement Base36 encoding in
    Kotlin.
     */
    private fun generateRandomString(idLength: Int = 5): String {
      val idBuilder: StringBuilder = StringBuilder()
      var randomIndex = 0
      for (i in 0 until idLength) {
        randomIndex = (Random.nextInt(until = characterSet.size))
        idBuilder.append(characterSet[randomIndex])
      }

      return idBuilder.toString()
    }
  }
}
